@@ -39,6 +39,11 @@ MATH1
 * ((<glpack>))
   * ((<gl_set_params>))
     Calls (({DCL.glpset})) multiple times (for each key and val of (({hash}))).
+MATH2
+* ((<shtlib>))
+  * ((<sht_get_n>)) calculate n (total wavenum)
+  * ((<sht_get_m>)) calcurate m (zonal wavenum)
+
 GRPH1
 * ((<sgpack>))
   * ((<sg_set_params>))
@@ -115,6 +120,26 @@ GRPH2
         ....
         DCLExt.gl_set_params(before)     # reset the change
 
+===shtlib
+---sht_get_n(mm)
+    calcurate n (total wavenum)
+
+    ARGUMENT
+    * mm (Integer) : truncation wavenumber
+
+    RETURN VALUE
+    * n (NArray) : an integer NArray contianing n's
+
+---sht_get_m(mm)
+
+    calcurate m (zonal wavenum)
+
+    ARGUMENT
+    * mm (Integer) : truncation wavenumber
+
+    RETURN VALUE
+    * m (NArray) : an integer NArray contianing m's
+
 ===sgpack
 ---sg_set_params(hash)
     Calls (({DCL.sgpset})) multiple times (for each key and val of (({hash}))).
@@ -668,6 +693,36 @@ module NumRu
 
     @@empty_hash = Hash.new
 
+    #<<< MATH2: shtlib >>>
+
+    # calcurate n (total wavenum)
+    def sht_get_n(mm)
+      ns = NArray.int((mm+1)**2)
+      mi = NArray.int(mm+1).indgen!
+      ns[0..mm] = mi  # for m=0
+      i = mm+1
+      for m in 1..mm
+        ns[i..(i+mm-m)] = mi[m..mm]
+        i += mm-m+1
+        ns[i..(i+mm-m)] = mi[m..mm]
+        i += mm-m+1
+      end
+      ns
+    end
+
+    # calcurate m (zonal wavenum)
+    def sht_get_m(mm)
+      ns = NArray.int((mm+1)**2)
+      i = mm+1
+      for m in 1..mm
+        ns[i..(i+mm-m)] = m
+        i += mm-m+1
+        ns[i..(i+mm-m)] = m
+        i += mm-m+1
+      end
+      ns
+    end
+
     #<<< udpack >>>
 
     def ud_coloring(clr_min=13, clr_max=99)
