@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*-
 require "narray"
 
 ############################################################
@@ -42,7 +43,7 @@ Module functions of Derivative Operater for NArray.
     * x (NArray): a NArray represents the dimension which derivative respect to.
       z.rank must be 1.
     * dim (Numeric): a Numeric represents the dimention which derivative respect to. 
-      you can give number count backward (((<dim>))<0), but ((<z.rank dim>)) must be > 0. 
+      you can give number count backward (((<dim>))<0), but ((<z.rank ＋dim>)) must be > 0. 
     * bc (Numeric) : a Numeric to represent boundary condition.
       Supported conditions are expressed in terms of boundary extension
       applied before differentiation:
@@ -69,7 +70,7 @@ Module functions of Derivative Operater for NArray.
       to. z.rank must be 1.
     * dim (Numeric): a Numeric represents the dimention which derivative 
       respect to. you can give number count backward (((<dim>))<0), but 
-      ((<z.rank dim>)) must be > 0. 
+      ((<z.rank ＋dim>)) must be > 0. 
     * bc (Numeric) : a Numeric to represent boundary condition.
       See ((<threepoint_O2nd_deriv>)) for supported conditions.
 
@@ -89,7 +90,7 @@ Module functions of Derivative Operater for NArray.
       to. z.rank must be 1.
     * dim (Numeric): a Numeric represents the dimention which derivative 
       respect to. you can give number count backward (((<dim>))<0), but 
-      ((<z.rank dim>)) must be > 0. 
+      ((<z.rank ＋dim>)) must be > 0. 
     * bc (Numeric) : a Numeric to represent boundary condition.
       See ((<threepoint_O2nd_deriv>)) for supported conditions.
 
@@ -109,7 +110,7 @@ Module functions of Derivative Operater for NArray.
     * z (NArray): a NArray which you want to expand boundary.
     * dim (Numeric): a Numeric represents the dimention which derivative 
       respect to. you can give number count backward (((<dim>))<0), but 
-      ((<z.rank dim>)) must be > 0. 
+      ((<z.rank ＋dim>)) must be > 0. 
 
     RETURN VALUE
     * expand_data (NArray): 
@@ -124,7 +125,7 @@ Module functions of Derivative Operater for NArray.
     * x (NArray): a NArray which you want to get difference.
     * dim (Numeric): a Numeric representing the dimention which derivative 
       respect to. you can give number count backward (((<dim>))<0), but 
-      ((<z.rank dim>)) must be > 0. 
+      ((<z.rank ＋dim>)) must be > 0. 
 
     RETURN VALUE
     * cdiff_data (NArray): (x_{i+1} - x_{i-1})
