@@ -10,7 +10,7 @@ require 'numru/ganalysis/beta_plane'
 module NumRu
   module GAnalysis
 
-    # QG_common: correction of common methods for QG, QG_sphere, and QG_sphere_div.
+    # QG_common: collection of common methods for QG, QG_sphere, and QG_sphere_div.
     module QG_common
       ## module_function  # disabled: module functions are specified one-by-one
 
@@ -218,7 +218,7 @@ module NumRu
 
     ######################################################
 
-    # Correction of common methods for QG_sphere and QG_sphere_div
+    # Collection of common methods for QG_sphere and QG_sphere_div
     module QG_sphere_common
 
       # Coriolis parameter f
@@ -248,6 +248,7 @@ module NumRu
       module_function
       extend QG_common
 
+      # This class is for internal use only
       class Uninitialized
         def method_missing(method_name)
           raise("Reference latitude has not been set. Call QG::set_lat0 to use the module QG.")
@@ -389,7 +390,7 @@ module NumRu
             p = p.convert2("Pa")
           end
         else
-          p = LogP.get_p(psi).convert_units("Pa")
+          p = Met.get_prs(psi).convert_units("Pa")
         end
         psi_xy = psi.threepoint_O2nd_deriv(0,bc,x).threepoint_O2nd_deriv(1,bc,y)
         psi_xx = psi.deriv2nd(0,bc,x)
