@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*-
 require "numru/gphys/varray"
 require "numru/gphys/varraynetcdf"
 require "numru/gphys/varraycomposite"
@@ -53,6 +54,15 @@ simulation. The 3rd is not physical and may not be considered as
     it returns a fancy but sometimes unwanted message if the 
     axis is the time with since field in the form of yyyy-mm-dd....    
 
+---Axis.join(ary)
+    Join multiple axis objects. (simple concatenation without sort)
+
+    ARGUMENTS
+    * ary :  Array (or 1D NArray) of Axis objects
+
+    RETURN VALUE
+    * an Axis
+
 ==Instance Methods
 ---name=(nm)
     Set a name.
@@ -372,10 +382,10 @@ module NumRu
 	 @cell
       end
       def cell_center?
-	 @cell && @pos.equal?(@cell_center)
+	 @cell && @pos.length == @cell_center.length
       end
       def cell_bounds?
-	 @cell && @pos.equal?(@cell_bounds)
+	 @cell && @pos.length == @cell_bounds.length
       end
       def bare_index?
 	 @bare_index
@@ -420,6 +430,51 @@ module NumRu
 	 out
       end
 
+      # Join multiple axis objects
+      # * ary :  Array (or 1D NArray) of Axis objects
+      def Axis.join(ary)
+        ax = ary[0]
+        #raise(ArgumentError,"not an Axis") if !ax.is_a?(Axis)
+        lens = ary.collect{|ax| ax.length}
+        na = ary.length
+
+        axpool = Hash.new
+        cell = ax.cell?
+        if cell
+          axpool[:celctrs] = ary.collect{|ax| ax.cell_center}
+          axpool[:celbnds] = ary.collect{|ax| ax.cell_bounds}
+        else
+          axpool[:pos] = ary.collect{|ax| ax.pos}
+        end
+        ax.aux_names.each do |nm|
+          axpool[nm] = ary.collect{|ax| ax.get_aux(nm)}
+        end
+        axpool.each do |nm,vas|
+          for i in 1...na
+            if ( vas[i-1].length == lens[i-1]+1 and vas[i].length == lens[i]+1 \
+                 and vas[i-1][-1].val == vas[i][0].val )
+              vas[i] = vas[i][1..-1]   # exclude overwrap (such as cell_bounds)
+            end
+            vas = NArray.to_na(vas) if vas.is_a?(Array)
+            axpool[nm] = VArrayComposite.new( vas )
+          end
+        end
+
+        bidx = ax.bare_index?
+        name = ax.name
+        jax = Axis.new(cell, bidx, name)   # prepare outout (jax: joined axis)
+        if cell
+          jax.set_cell(axpool.delete(:celctrs), axpool.delete(:celbnds))
+          ax.cell_center? ? jax.set_pos_to_center : jax.set_pos_to_bounds
+        else
+          jax.set_pos(axpool.delete(:pos))
+        end
+        axpool.each do |nm,va|
+          jax.set_aux(nm, va)
+        end
+        jax
+      end
+
       def collect
 	 # This method is like ((<copy>)), but it is the 'collect' 
          # iterator for each VArray in self (block required).
@@ -670,12 +725,116 @@ module NumRu
       end
 
       def cut(coord_cutter)
-	_cut_(false, coord_cutter)
+        if cyclic_extendible?
+          _cut_cyclic(false, coord_cutter)
+        else
+          _cut_(false, coord_cutter)
+        end
       end
       def cut_rank_conserving(coord_cutter)
-	_cut_(true, coord_cutter)
+        if cyclic_extendible?
+          _cut_cyclic(true, coord_cutter)
+        else
+          _cut_(true, coord_cutter)
+        end
       end
 
+      def _cut_cyclic(conserve_rank, coord_cutter)
+
+	# assume that the coordinates are monotonic (without checking)
+
+	slicer = Array.new
+	ax = self.pos.val
+        modulo = self.modulo
+        units = self.pos.get_att('units')
+        calendar = self.pos.get_att('calendar')
+
+	case coord_cutter
+	when true
+	  slicer=true
+          newax = self
+	when Range
+	  # find the grid points included in the range
+	  range = coord_cutter
+	  range_ary = [range.first, range.last]
+	  r0 = range_ary.min  # called xmin in _cut_
+	  r1 = range_ary.max  # called xmax in _cut
+          r0 = UNumeric.from_date(r0,units,calendar).val if r0.class <= Date
+          r1 = UNumeric.from_date(r1,units,calendar).val if r1.class <= Date
+          x0 = ax.min
+          x1 = ax.max
+          na = ( (r0 - x1)/modulo ).ceil
+          nb = ( (r1 - x0)/modulo ).floor
+          if na > nb
+	    raise "Range #{range} does not include any grid point in the axis '#{name}'"
+          elsif na==nb
+            # no need for cyclic repetition
+            n = na
+            wh = ( (ax >= r0-n*modulo) & (ax <= r1-n*modulo) ).where
+            if wh.length == 0
+              raise "Range #{range} does not include any grid point in the axis '#{name}'"
+            end
+            slicer = wh.min..wh.max
+            newax = self[slicer]
+            if n != 0
+              newax = newax.add( n*modulo )  # shift the axis to match the range
+            end
+          else   # na < nb
+            # cyclic repetition needed
+            slicer = Array.new
+            shift = Array.new
+            ns = (na..nb).to_a
+            ns.reverse! if ax[0] > ax[-1]  # to keep mononicity
+            for n in ns
+              wh = ( (ax >= r0-n*modulo) & (ax <= r1-n*modulo) ).where
+              slicer += wh.to_a
+              shift += [n*modulo] * wh.length
+            end
+            newax = self[slicer].add( shift )
+          end
+	when Numeric, Date, DateTime
+	  # find the nearst point
+	  pt = coord_cutter
+          pt = UNumeric.from_date(pt,units,calendar).val if pt.class <= Date
+	  dx, base = _cyclic_dx(ax-pt, modulo)
+	  minloc = _minloc_(dx)
+	  if conserve_rank
+	    slicer=minloc..minloc
+	  else
+	    slicer=minloc
+	  end
+          shift = -base[slicer]
+          newax = self[slicer]  # Don't worry. It shouldn't be an Axis any more
+	when Array, NArray
+	  # find the nearst points
+	  ary = coord_cutter
+	  slicer = Array.new
+	  shift = Array.new
+          ary.each{ |pt|
+            pt = UNumeric.from_date(pt,units,calendar).val if pt.class <= Date
+	    dx, base = _cyclic_dx(ax-pt, modulo)
+	    minloc = _minloc_(dx)
+            slicer.push( minloc )
+#            p "****",base,slicer,"#",base[slicer]
+            shift.push( -base[minloc] )
+	  }
+          newax = self[slicer].copy
+          newax.set_pos( newax.pos + NArray.to_na(shift) )
+	else
+	  raise TypeError, "(#{coord_cutter.inspect}) is not accepted as a coordinate selector"
+	end
+
+	[ newax, slicer ]
+      end
+      private :_cut_cyclic
+
+      def _cyclic_dx(dx, modulo)
+        base = modulo*(dx/modulo).round
+        dx = ( dx - base ).abs
+        [dx, base]
+      end
+      private :_cyclic_dx
+
       def _cut_(conserve_rank, coord_cutter)
 
 	# assume that the coordinates are monotonic (without checking)
@@ -735,7 +894,7 @@ module NumRu
         # Developper's MEMO: Such a method should be supported in NArray
         minloc = 0  # initialization
         for i in 0...(a.length-1)   # here, do not assume monotonic
-	  minloc = i+1 if a[i+1]<a[i]
+	  minloc = i+1 if a[i+1]<a[minloc]
 	end
 	minloc
       end
@@ -826,6 +985,35 @@ module NumRu
 	@pos.axis_cyclic_extendible?
       end
 
+      def add(z)
+        self.copy.add!(z)
+      end
+
+      # * z : usually a scalar, but can be an array-like object (which 
+      #   may be used in cyclic operations)
+      def add!(z)
+        @pos += z
+
+        if @cell
+          if cell_bounds?
+            if z.respond_to?(:length)
+              @cell_center += z[0..-2]
+            else
+              @cell_center += z
+            end
+          elsif cell_center?
+            if z.respond_to?(:length)  # array-like
+              @cell_bounds[0..-2] += z
+              @cell_bounds[-1..-1] += z[-1..-1]   # simply use the last one
+            else
+              @cell_bounds += z
+            end
+          end
+        end
+
+        self
+      end
+
       #########  private methods ####################################
 
       private
@@ -840,9 +1028,9 @@ module NumRu
 	 # < define numerical integration / averaging >
 
 	 ## 2005/09/12 DEVLOPMENT MEMO by horinout
-	 ## cyclic ł傤ǂPObhL΂ modulo ɒBꍇ
-	 ## _TvZ^CvłTCNbNpd݂
-         ## 悤ɂD
+	 ## cyclic でちょうど１グリッド伸ばしたら modulo に達する場合
+	 ## 点サンプルやセル中央タイプでもサイクリック用重みをつける
+         ## ようにしたい．
 
 	 if( @bare_index )
 	    @integ_weight = NArray.int(@pos.length).fill!(1)
@@ -960,4 +1148,17 @@ if $0 == __FILE__
 
    axpt.pos.set_att("units","days since 2001-01-01")
    p axpt.cut(DateTime.new(2001,1,4,12,0))
+
+   p "## test join"
+   axpt.set_aux("AUX1", xc*0.1 )
+   axpt2 = Axis.new().set_pos(xc+10)
+   axpt2.set_aux("AUX1", xc*0.01 )
+   jax = Axis.join([axpt,axpt2])
+   p jax, jax.name, jax.pos.val.to_a, jax.get_aux("AUX1").val.to_a
+
+   p "## test join (cell)"
+   axcel_c2 = Axis.new(true).set_cell(xc+10,xb+10)
+   axcel_c2.set_pos_to_center
+   jax = Axis.join([axcel_c,axcel_c2])
+   p jax, jax.name, jax.pos.val.to_a, jax.cell_bounds.val.to_a
 end
