@@ -72,13 +72,13 @@ module NumRu
       module_function
 
       def is_a_Grib?(filename)
-        Grib.is_a_Grib?(filename)
+        VArrayGrib.grib.is_a_Grib?(filename)
       end
 
       def open(file, varname)
         if file.is_a?(String)
-          file = Grib.open(file)
-        elsif ! Grib===file
+          file = VArrayGrib.grib.open(file)
+        elsif ! VArrayGrib.grib===file
           raise ArgumentError, "1st arg must be a Grib or a file name"
         end
 
@@ -157,8 +157,8 @@ module NumRu
         opened = false
         case file
         when String
-          file = Grib.open(file)
-        when Grib
+          file = VArrayGrib.grib.open(file)
+        when VArrayGrib.grib
           opened = true
         else
           raise ArgumentError, "arg must be a Grib or a file name"
@@ -197,7 +197,7 @@ if $0 == __FILE__
 
   puts "\n** test convert (T.jan.nc to tmp.grib) **"
   temp3 = GPhys::NetCDF_IO.open("../../../testdata/T.jan.nc","T")
-  file = Grib.create("tmp.grib")
+  file = VArrayGrib.grib.create("tmp.grib")
   GPhys::Grib_IO.write(file,temp3)
   file.close
 
