@@ -598,12 +598,12 @@ EOS
       # and determine the record number in the target file
 
       init_date = DateTime.new(init_year,init_month,init_day,init_hour,init_min)
+      init_time = UNumeric.from_date(init_date,units,calendar).val
 
       init_t = t
       while init_t > 0
         time = ary_time[init_t-1]
-        date = (UNumeric[time,units.to_s]).to_datetime(0.1,calendar)
-        break if date < init_date
+        break if time < init_time
         init_t = init_t - 1
       end
 
@@ -1051,6 +1051,14 @@ EOS
       rescue StandardError,NameError
         raise "Cannot convert hour or time into interger: "+stime
       end
+    elsif ( /\d\d/ =~ stime )
+      begin
+        shour = stime
+        hour = shour.to_i
+        min = 0
+      rescue StandardError,NameError
+        raise "Cannot convert hour or time into interger: "+stime
+      end
     else
       hour = 0
       min = 0
@@ -1207,56 +1215,58 @@ EOS
 
  class GrADSVar
 
-  def initialize(file,varname)
+  def initialize(file,varname,varonly=false,dimonly=false)
     @varname = varname
 
     if file.is_a?(String)
       file = GrADS_Gridded.open(file)
-    elsif ! file.is_a?(GrADS_Gridded)
+    elsif !file.is_a?(GrADS_Gridded)
       raise ArgumentError, "1st arg must be a GrADS_Gridded or a file name"
     end
     @ctl = file  # control file name
-#    @ctl = ctl
-
-    @idim = -1
-    for i in 0...@ctl.dimensions.length
-#      p @ctl.dimensions[i][:name]
-      if @varname == @ctl.dimensions[i][:name]
-        @idim = i
-      end
-    end
 
     @attr = NumRu::Attribute.new
-    if( @idim != -1 )
-      @dimensions = [ @ctl.dimensions[@idim] ]
-      @rank = 1
-      @attr[:name] = @dimensions[0][:name]
-      @attr[:long_name] = @dimensions[0][:description]
-      @attr[:units] = @dimensions[0][:units]
-      if @attr[:long_name] == "time" && @ctl.get_att("365_day_calendar") == true
-        @attr[:calendar] = "365_day"
-      end
 
-      @shape = [@dimensions[0][:len]]
-    else
+    found = false
+    if !dimonly
       @dimensions = @ctl.dimensions
       @rank = @dimensions.length
-      found = false
-      for i in 0...@ctl.variables.length
-        if @varname == @ctl.variables[i][:name]
-          @attr[:long_name] = @ctl.variables[i][:description]
-          @attr[:nlev] = @ctl.variables[i][:nlev].to_s
+      @ctl.variables.each do |var|
+        if @varname == var[:name]
+          @idim = -1
+          @attr[:long_name] = var[:description]
+          @attr[:nlev] = var[:nlev].to_s
 
           @shape = [@dimensions[0][:len],@dimensions[1][:len],
                     @dimensions[2][:len],@dimensions[3][:len]]
-          @shape[2] = @ctl.variables[i][:nlev].to_i if( @ctl.variables[i][:nlev] )
+          @shape[2] = var[:nlev].to_i if( var[:nlev] )
 
           @attr[:missing_value] = NArray.sfloat(1).fill!(@ctl.undef)
           found = true
         end
       end
-      raise "variable #{@varname} is not found" if !found
     end
+
+    if !found && !varonly
+      @ctl.dimensions.each_with_index do |dim,i|
+        if @varname == dim[:name]
+          @idim = i
+          @dimensions = [ dim ]
+          @rank = 1
+          @attr[:name] = dim[:name]
+          @attr[:long_name] = dim[:description]
+          @attr[:units] = dim[:units]
+          if @attr[:long_name] == "time" && @ctl.get_att("365_day_calendar") == true
+            @attr[:calendar] = "365_day"
+          end
+
+          @shape = [dim[:len]]
+          found = true
+        end
+      end
+    end
+
+    raise "variable #{@varname} is not found" if !found
   end
 
   attr_accessor (:rank)
