@@ -262,7 +262,7 @@ PARAM_RDN_LWTOP = 114
 PARAM_RDN_LW = 115
 PARAM_RDN_SW = 116
 PARAM_RDN_GLBL = 117
-#PARAM_BRTMP = 118
+PARAM_BRTMP = 118
 #PARAM_LWRAD = 119
 #PARAM_SWRAD = 120
 PARAM_LAT_HT = 121
@@ -311,6 +311,7 @@ PARAM_CLWMR = 2153
 PARAM_O3MR = 2154
 PARAM_CIN = 2156
 PARAM_CAPE = 2157
+PARAM_VBDSF = 2166
 PARAM_HPBL = 2221
 PARAM_5WAVH = 2222
 PARAM_5WAVA = 2230
@@ -528,7 +529,7 @@ NAMES_UNITS = {
   PARAM_PTND => ["Pressure tendency", "PTEND", "Pa/s", "tendency_of_air_pressure"],
 #  PARAM_ICAHT => nil,
   PARAM_GPT => ["Geopotential", "GP", "m2/s2", "geopotential"],
-  PARAM_GPT_HGT => ["Geopotential height", "HGTZ", "Gpm", "geopotential_height"],
+  PARAM_GPT_HGT => ["Geopotential height", "HGT", "Gpm", "geopotential_height"],
   PARAM_GEOM_HGT => ["Geometric height", "DIST", "M", "altitude"],
   PARAM_HSTDV => ["Standard deviation of height", "HSTDV", "M", "equivalent_thickness_at_stp_of_atmosphere_o3_content"],
   PARAM_TOZNE => ["Total ozone", "TOZNE", "Dobson", "equivalent_thickness_at_stp_of_atmosphere_o3_content"],
@@ -639,7 +640,7 @@ NAMES_UNITS = {
   PARAM_RDN_LW => ["Long wave radiation", "LWAVR", "W/m2", "net_upward_longwave_flux_in_air"],
   PARAM_RDN_SW => ["Short wave radiation", "SWAVR", "W/m2", "net_upward_shortwave_flux_in_air"],
   PARAM_RDN_GLBL => ["Global radiation", "GRAD", "W/m2", "surface_downwelling_shortwave_flux"],
-#  PARAM_BRTMP => nil,
+  PARAM_BRTMP => ["Brightness temperature", "BRTMP", "K", "brightness_temperature"],
 #  PARAM_LWRAD => nil,
 #  PARAM_SWRAD => nil,
   PARAM_LAT_HT => ["Latent heat net flux", "LHTFL", "W/m2", "surface_upward_latent_heat_flux"],
@@ -718,7 +719,7 @@ NAMES_UNITS = {
 #163	CSDLF	W/m2		Clear Sky downward long wave flux		
 #164	CFNSF	W/m2		Cloud forcing net solar flux		
 #165	CFNLF	W/m2		Cloud forcing net long wave flux		
-#166	VBDSF	W/m2		Visible beam downward solar flux		
+  PARAM_VBDSF => ["Visible beam downward solar flux", "VBDSF", "W/m2", nil],
 #167	VDDSF	W/m2		Visible diffuse downward solar flux		
 #168	NBDSF	W/m2		Near IR beam downward solar flux		
 #169	NDDSF	W/m2		Near IR diffuse downward solar flux		
@@ -1420,6 +1421,7 @@ PARAMS_JMA = {
   162 => PARAM_CSULF,
   163 => PARAM_CSDLF,
   165 => PARAM_ADVVA,
+  166 => PARAM_VBDSF,
   168 => PARAM_FRQP,
   169 => PARAM_FRQC,
   173 => PARAM_GWDUA,
@@ -1461,3 +1463,26 @@ PARAMS_JMA = {
   251 => PARAM_LWHR,
   253 => PARAM_LRGMR,
 }
+
+## JRA parameter code table
+## JRA-55: Code Table 200: 
+## http://rda.ucar.edu/datasets/ds628.0/#metadata/grib.html?_do=y
+##   Ref JRA-25: 
+##   http://rda.ucar.edu/datasets/ds625.0/docs/jra-25_grib_table.html
+##   POT_VOR is also 4 in JRA-25. Otherwise, JRA-25 is supported by
+##   the PARAMS_JMA table (as of 2014-08)
+PARAMS_JRA55 = PARAMS_2.merge( PARAMS_JMA )
+PARAMS_JRA55[4]   = PARAM_POT_VOR
+PARAMS_JRA55[118] = PARAM_BRTMP  # 118 is mean surface pressure in JRA-25
+PARAMS_JRA55[221] = PARAM_CLWMR  
+PARAMS_JRA55[225] = PARAM_SOILW
+=begin
+Yet to be supported (JRA-55)
+190	UTHE	Zonal thermal energy flux	W m-1
+191	VTHE	Meridional thermal energy flux	W m-1
+226	SMC	Mass concentration of condensed water in soil	kg m-3
+227	CW	Cloud liquid water	kg m-2
+228	CLWC	Cloud liquid water	kg kg-1
+229	CIWC	Cloud ice	kg kg-1
+252	VTYPE	Type of vegetation	code
+=end
