@@ -328,10 +328,12 @@ module NumRu
         nms = s.split(/ +/)
         case data.file
         when NArray
-          nms.delete_if{|nm| !data.file[0].var(nm)}
+          fl = data.file[0]
         else
-          nms.delete_if{|nm| !data.file.var(nm)}
+          fl = data.file
         end
+        nms.delete_if{|nm| !fl.var(nm)}
+        nms.delete_if{|nm| fl.dim_names.include?(nm)}
         nms
       else
         nil
