@@ -4,6 +4,8 @@
 #   See ./druby_serv1.rb
 
 require "drb/drb"
+require "numru/gphys"
+include NumRu
 
 DRb.start_service
 uri = ARGV.shift || raise("Usage: % #{$0} uri")
